using System;
using System.Windows;

namespace Infragistics.Samples.Shared.Models
{
    /// <summary>
    /// Represents a model that holds basic information about a Theme (ResourceDictionary)
    /// </summary>
    public class Theme
    {
        /// <summary>
        /// Gets or sets an index of theme 
        /// </summary>
        public string ThemeId { get; set; }
        /// <summary>
        /// Gets or sets the name of theme 
        /// </summary>
        public string ThemeName { get; set; }
        /// <summary>
        /// Gets or sets the location of a resource dictionary with the theme 
        /// </summary>
        public Uri ThemeUri { get; set; }
        /// <summary>
        /// Gets whether or not this is default theme for a control
        /// <para>Returns true if ThemeUri is null, otherwise it returns false.</para>
        /// </summary>
        public bool IsDefaultTheme
        {
            get { return this.ThemeUri == null; }
        }

        public Theme()
        {
            this.ThemeUri = null;
        }

        public Theme(string idx, string name, Uri uri)
        {
            this.ThemeId = idx;
            this.ThemeName = name;
            this.ThemeUri = uri;
        }

        public override string ToString()
        {
            return this.ThemeName;
        }
        //NOTE: use ThemeResourceHelper to add/remove ResourceDictionary
        //public static void RemoveThemesForControl(ResourceDictionary dicts, string controlName)
        //{
        //    for (int i = dicts.MergedDictionaries.Count - 1; i >= 0; i--)
        //    {
        //        var rd = dicts.MergedDictionaries[i];
        //        if (rd.Source.OriginalString.Contains(controlName))
        //        {
        //            dicts.MergedDictionaries.RemoveAt(i);
        //        }
        //    }
        //}

    }
}