using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;

namespace Infragistics.Samples.Shared.Tools
{
    /// <summary>
    /// Provides useful methods when working with Theme resource dictionary
    /// </summary>
    public class ThemeResourceHelper
    {
        /// <summary>
        /// Removes ResourceDictionary from the list of ResourceDictionary objects the gived ResourceDictionary's MergedDictionaries,
        /// which has the specified string in their names.
        /// </summary>
        public static void RemoveResourceDictionariesForControl(ResourceDictionary dicts, string controlName)
        {
            for (int i = dicts.MergedDictionaries.Count - 1; i >= 0; i--)
            {
                var rd = dicts.MergedDictionaries[i];
                if (rd.Source!=null && rd.Source.OriginalString.Contains(controlName))
                {
                    dicts.MergedDictionaries.RemoveAt(i);
                }
            }
        }

        public static void SetOffice2013Theme(FrameworkElement container, string controlName)
        {
            var themePath = String.Format("/Infragistics.Themes.Office2013Theme;component/Office2013.{0}.xaml", controlName);
            var themeUri = new Uri(themePath, UriKind.Relative);

            RemoveResourceDictionariesForControl(container.Resources, controlName);

            container.Resources.MergedDictionaries.Add(new ResourceDictionary() { Source = themeUri });
        }
    }
}